# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-05-03 03:52
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('qeventlog', '0002_qtaskstate_task'),
    ]

    operations = [
        migrations.CreateModel(
            name='QTaskName',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, db_index=True, max_length=128, null=True, unique=True)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.RemoveField(
            model_name='qtaskstate',
            name='task',
        ),
        migrations.AddField(
            model_name='qtaskstate',
            name='name',
            field=models.ForeignKey(default=None, on_delete=django.db.models.deletion.CASCADE, to='qeventlog.QTaskName'),
            preserve_default=False,
        ),
    ]
